(ns web-client.core
    (:require [reagent.core :as reagent :refer [atom]]
              [reagent.session :as session]
              [secretary.core :as secretary :include-macros true]
              [accountant.core :as accountant]
              [com.stuartsierra.component :as component]))

;; -------------------------
;; Views

(defrecord AppView [root-selector
                    home-page
                    about-page
                    current-page
                    root]
 component/Lifecycle
 (start [component]
   (update component
           :root #(or % (do (accountant/configure-navigation!)
                            (accountant/dispatch-current!)
                            (let [root-el (.getElementById js/document root-selector)
                                  root (reagent/render [current-page] root-el)]
                                 (assoc component :root root))))))
 (stop [component]
       (reagent/unmount-component-at-node (reagent/dom-node (:root component)))
       (dissoc component :root)))

(defn home-page []
  [:div [:h2 "Welcome to web-client"]
   [:div [:a {:href "/about"} "go to about page"]]])

(defn about-page []
  [:div [:h2 "About web-client"]
   [:div [:a {:href "/"} "go to the home page"]]])

(defn current-page []
  [:div [(session/get :current-page)]])

;; -------------------------
;; Routes

(secretary/defroute "/" []
  (session/put! :current-page #'home-page))

(secretary/defroute "/about" []
  (session/put! :current-page #'about-page))

;; -------------------------
;; Initialize app

(defonce system (atom (map->AppView {:root-selector "app"
                                     :home-page home-page
                                     :about-page about-page
                                     :current-page current-page})))

(defn init! []
      (swap! system update component/start))

(defn reload! []
      (swap! system update component/stop)
      (init!))